@@ -1,5 +1,5 @@
 # -*- cperl -*-
-# Copyright (c) 2004, 2017, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2004, 2018, Oracle and/or its affiliates. All rights reserved.
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@ our @EXPORT= qw(report_option mtr_print_line mtr_print_thick_line
 use mtr_match;
 use File::Spec;
 use My::Platform;
-use POSIX qw[ _exit ];
+use POSIX qw(_exit floor);
 use IO::Handle qw[ flush ];
 require "mtr_io.pl";
 use mtr_results;
@@ -40,6 +40,7 @@ use mtr_results;
 my $tot_real_time= 0;
 
 my $done_percentage= 0;
+my $tests_completed= 0;
 
 our $timestamp= 0;
 our $timediff= 0;
@@ -67,9 +68,11 @@ sub _name {
   return $name ? $name." " : undef;
 }
 
-sub _mtr_report_test_name ($)
+sub _mtr_report_test_name ($$)
 {
   my $tinfo= shift;
+  my $done_percentage= shift;
+
   my $tname= $tinfo->{name};
 
   return unless defined $verbose;
@@ -141,13 +144,13 @@ sub mtr_report_test ($) {
   {
     if ($tinfo->{'name'} && !$retry)
     {
-      $::remaining= $::remaining - 1;
-      $done_percentage = 100 - int (($::remaining * 100) /
-                                    ($::num_tests_for_report));
+      $tests_completed= $tests_completed + 1;
+      $done_percentage=
+        floor(($tests_completed / $::num_tests_for_report) * 100);
     }
   }
 
-  my $test_name = _mtr_report_test_name($tinfo);
+  my $test_name = _mtr_report_test_name($tinfo, $done_percentage);
 
   if ($result eq 'MTR_RES_FAILED'){
 
