@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -80,9 +80,14 @@ static MYSQL_SYSVAR_STR(
   update_keyring_file_data,                                    /* update()   */
   MYSQL_DEFAULT_KEYRINGFILE                                    /* default    */
 );
+static MYSQL_SYSVAR_BOOL(open_mode, keyring_open_mode,
+                         PLUGIN_VAR_INVISIBLE | PLUGIN_VAR_RQCMDARG,
+                         "Mode in which keyring file should be opened", NULL,
+                         NULL, TRUE);
 
 static struct st_mysql_sys_var *keyring_file_system_variables[]= {
   MYSQL_SYSVAR(data),
+  MYSQL_SYSVAR(open_mode),
   NULL
 };
 
@@ -94,6 +99,8 @@ static int keyring_init(MYSQL_PLUGIN plugin_info)
     keyring_init_psi_keys();
 #endif
 
+    DBUG_EXECUTE_IF("simulate_keyring_init_error", return TRUE;);
+
     if (init_keyring_locks())
       return TRUE;
 
@@ -103,7 +110,7 @@ static int keyring_init(MYSQL_PLUGIN plugin_info)
       logger->log(MY_ERROR_LEVEL, "Could not create keyring directory "
         "The keyring_file will stay unusable until correct path to the keyring "
         "directory gets provided");
-      return FALSE;
+      return TRUE;
     }
     keys.reset(new Keys_container(logger.get()));
     IKeyring_io *keyring_io= new Buffered_file_io(logger.get());
@@ -115,7 +122,7 @@ static int keyring_init(MYSQL_PLUGIN plugin_info)
         " can be created in the specified location. "
         "The keyring_file will stay unusable until correct path to the keyring file "
         "gets provided");
-      return FALSE;
+      return TRUE;
     }
     is_keys_container_initialized = TRUE;
     return FALSE;
