@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #include <climits>
 #include <set>
 #include <limits>
+#include <assert.h>
 
 /**
   6 is the recommended value. Too large numbers
@@ -44,6 +45,17 @@ static const int XCOM_MAX_HANDLERS= 6;
 */
 static const uint64_t WAITING_TIME= 30;
 
+/*
+  Number of attempts to join a group.
+*/
+static const unsigned int JOIN_ATTEMPTS= 0;
+
+/*
+  Sleep time between attempts defined in seconds.
+*/
+static const uint64_t JOIN_SLEEP_TIME= 5;
+
+
 Gcs_xcom_utils::~Gcs_xcom_utils() {}
 
 u_long Gcs_xcom_utils::build_xcom_group_id(Gcs_group_identifier &group_id)
@@ -53,16 +65,6 @@ u_long Gcs_xcom_utils::build_xcom_group_id(Gcs_group_identifier &group_id)
 }
 
 
-std::string *Gcs_xcom_utils::build_xcom_member_id(const std::string &address)
-{
-  std::ostringstream string_builder;
-
-  string_builder << address.c_str();
-
-  return new std::string(string_builder.str());
-}
-
-
 void
 Gcs_xcom_utils::
 process_peer_nodes(const std::string *peer_nodes,
@@ -594,11 +596,10 @@ Gcs_xcom_proxy_impl::Xcom_handler::~Xcom_handler()
 }
 
 
-node_address *Gcs_xcom_proxy_impl::new_node_address(unsigned int n,
-                                                    char *names[])
+node_address *Gcs_xcom_proxy_impl::new_node_address_uuid(
+  unsigned int n, char *names[], blob uuids[])
 {
-  //Xcom will change n's type to unsigned later in a separate patch.
-  return ::new_node_address(static_cast<int>(n), names);
+  return ::new_node_address_uuid(static_cast<int>(n), names, uuids);
 }
 
 
@@ -787,7 +788,7 @@ Gcs_xcom_proxy_impl::xcom_wait_for_xcom_comms_status_change(int& status)
   if (res != 0)
   {
     // There was an error
-    status= XCOM_COMMS_ERROR;
+    status= XCOM_COMMS_OTHER;
 
     if(res == ETIMEDOUT)
     {
@@ -891,18 +892,38 @@ Gcs_xcom_proxy_impl::xcom_client_force_config(node_list *nl,
 }
 
 Gcs_xcom_nodes::Gcs_xcom_nodes(const site_def *site, node_set &nodes)
-  : m_node_no(site->nodeno), m_addresses(), m_statuses(), m_size(nodes.node_set_len)
+  : m_node_no(site->nodeno), m_addresses(), m_uuids(), m_statuses(),
+    m_size(nodes.node_set_len)
 {
+  Gcs_uuid uuid;
   for (unsigned int i= 0; i < nodes.node_set_len; ++i)
   {
+    /* Get member address and save it. */
     std::string address(site->nodes.node_list_val[i].address);
     m_addresses.push_back(address);
+
+    /* Get member uuid and save it. */
+    uuid.decode(
+      reinterpret_cast<uchar *>(site->nodes.node_list_val[i].uuid.data.data_val),
+      site->nodes.node_list_val[i].uuid.data.data_len
+    );
+    m_uuids.push_back(uuid);
+
+    /* Get member status and save it */
     m_statuses.push_back(nodes.node_set_val[i] ? true: false);
   }
   assert(m_size == m_addresses.size());
   assert(m_size == m_statuses.size());
 }
 
+
+Gcs_xcom_nodes::Gcs_xcom_nodes()
+  : m_node_no(0), m_addresses(), m_uuids(), m_statuses(),
+    m_size(0)
+{
+}
+
+
 unsigned int Gcs_xcom_nodes::get_node_no() const
 {
    return m_node_no;
@@ -913,6 +934,25 @@ const std::vector<std::string> &Gcs_xcom_nodes::get_addresses() const
   return m_addresses;
 }
 
+const std::vector<Gcs_uuid> &Gcs_xcom_nodes::get_uuids() const
+{
+  return m_uuids;
+}
+
+
+const Gcs_uuid *Gcs_xcom_nodes::get_uuid(const std::string &address) const
+{
+  for (size_t index= 0; index < m_size; index++)
+  {
+    if (!m_addresses[index].compare(address))
+    {
+      return &m_uuids[index];
+    }
+  }
+  return NULL;
+}
+
+
 const std::vector<bool> &Gcs_xcom_nodes::get_statuses() const
 {
   return m_statuses;
@@ -966,7 +1006,10 @@ fix_parameters_syntax(Gcs_interface_parameters &interface_params)
     interface_params.get_parameter("wait_time"));
   std::string *ip_whitelist_str= const_cast<std::string *>(
     interface_params.get_parameter("ip_whitelist"));
-
+  std::string *join_attempts_str= const_cast<std::string *>(
+    interface_params.get_parameter("join_attempts"));
+  std::string *join_sleep_time_str= const_cast<std::string *>(
+    interface_params.get_parameter("join_sleep_time"));
 
   // sets the default value for compression (ON by default)
   if (!compression_str)
@@ -1019,6 +1062,22 @@ fix_parameters_syntax(Gcs_interface_parameters &interface_params)
 
     interface_params.add_parameter("ip_whitelist", iplist);
   }
+
+  // sets the default join attempts
+  if (!join_attempts_str)
+  {
+    std::stringstream ss;
+    ss << JOIN_ATTEMPTS;
+    interface_params.add_parameter("join_attempts", ss.str());
+  }
+
+  // sets the default sleep time between join attempts
+  if (!join_sleep_time_str)
+  {
+    std::stringstream ss;
+    ss << JOIN_SLEEP_TIME;
+    interface_params.add_parameter("join_sleep_time", ss.str());
+  }
 }
 
 static enum_gcs_error
@@ -1063,6 +1122,10 @@ is_parameters_syntax_correct(const Gcs_interface_parameters &interface_params)
     interface_params.get_parameter("compression");
   const std::string *wait_time_str=
     interface_params.get_parameter("wait_time");
+  const std::string *join_attempts_str=
+    interface_params.get_parameter("join_attempts");
+  const std::string *join_sleep_time_str=
+    interface_params.get_parameter("join_sleep_time");
 
   /*
     -----------------------------------------------------
@@ -1222,6 +1285,26 @@ is_parameters_syntax_correct(const Gcs_interface_parameters &interface_params)
     goto end;
   }
 
+  if(join_attempts_str &&
+     (join_attempts_str->size() == 0 ||
+      !is_number(*join_attempts_str)))
+  {
+    MYSQL_GCS_LOG_ERROR("The join_attempts parameter ("
+                        << join_attempts_str << ") is not valid.")
+    error= GCS_NOK;
+    goto end;
+  }
+
+  if(join_sleep_time_str &&
+     (join_sleep_time_str->size() == 0 ||
+      !is_number(*join_sleep_time_str)))
+  {
+    MYSQL_GCS_LOG_ERROR("The join_sleep_time parameter ("
+                        << join_sleep_time_str << ") is not valid.")
+    error= GCS_NOK;
+    goto end;
+  }
+
 end:
   return error == GCS_NOK ? false : true;
 }
