@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -45,7 +45,9 @@ struct site_def_ptr_array {
 };
 typedef struct site_def_ptr_array site_def_ptr_array;
 
-define_xdr_funcs(site_def_ptr)
+init_xdr_array(site_def_ptr)
+free_xdr_array(site_def_ptr)
+set_xdr_array(site_def_ptr)
 
 /* FIFO of site definitions */
 static site_def_ptr_array site_defs;
@@ -460,7 +462,7 @@ void import_config(gcs_snapshot *gcs_snap)
 			    cp->nodes.node_list_val, site);
 			site->start = cp->start;
 			site->boot_key = cp->boot_key;
-			site_install_action(site);
+			site_install_action(site, app_type);
 		}
 	}
 }
