@@ -1,4 +1,4 @@
-/* Copyright (c) 2012, 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@
 
 #include <limits.h>
 #include <stdlib.h>
-#include "xcom_proto_enum.h"
+#include "xcom_vp.h"
 #include "node_connection.h"
 
 #ifndef WIN
@@ -61,11 +61,12 @@
 #include "task_os.h"
 #include "task.h"
 #include "xcom_cfg.h"
-#ifndef USE_SELECT
+#ifndef _WIN32
 #include <sys/poll.h>
 #endif
 
 #include "retry.h"
+#include "xdr_utils.h"
 
 extern char *pax_op_to_str(int x);
 
@@ -74,26 +75,39 @@ task_arg null_arg = {a_end,{0}};
 struct iotasks ;
 typedef struct iotasks iotasks;
 
-#ifdef USE_SELECT
+typedef struct {
+  u_int pollfd_array_len;
+  pollfd *pollfd_array_val;
+} pollfd_array;
+
+typedef task_env* task_env_p;
+
+typedef struct {
+  u_int task_env_p_array_len;
+  task_env_p *task_env_p_array_val;
+} task_env_p_array;
+
+init_xdr_array(pollfd)
+free_xdr_array(pollfd)
+set_xdr_array(pollfd)
+get_xdr_array(pollfd)
+init_xdr_array(task_env_p)
+free_xdr_array(task_env_p)
+set_xdr_array(task_env_p)
+get_xdr_array(task_env_p)
+
 struct iotasks {
-	int	maxfd;
-	fd_set read_set;
-	fd_set write_set;
-	fd_set err_set;
-	linkage tasks; /* OHKFIX Should be one each for read and write */
+  int nwait;
+  pollfd_array fd;
+  task_env_p_array tasks;
 };
-#else
-struct iotasks {
-	int	nwait;
-	struct pollfd fd[MAXFILES];
-	task_env * tasks[MAXFILES];
-};
-#endif
+
 int	task_errno = 0;
 static task_env *extract_first_delayed();
 static task_env *task_ref(task_env *t);
 static task_env *task_unref(task_env *t);
 static void wake_all_io();
+static void task_sys_deinit();
 
 /* Return time as seconds */
 static double	_now = 0.0;
@@ -557,6 +571,7 @@ static void task_delete(task_env *t)
 #if 1
 	free(deactivate(t)); /* Deactivate and free task */
 #else
+  deactivate(t);
 	link_into(&t->l, &free_tasks);
 #endif
 	active_tasks--;
@@ -598,14 +613,13 @@ task_env *task_deactivate(task_env *t)
 
 
 /* Set terminate flag and activate task */
-task_env *task_terminate(task_env *t)
-{
-	if (t) {
-		MAY_DBG(FN; PTREXP(t); STREXP(t->name));
-		t->terminate = KILL; /* Set terminate flag */
-		activate(t); /* and get it running */
-	}
-	return t;
+task_env *task_terminate(task_env *t) {
+  if (t) {
+    DBGOUT(FN; PTREXP(t); STREXP(t->name); NDBG(t->refcnt, d));
+    t->terminate = KILL; /* Set terminate flag */
+    activate(t);         /* and get it running */
+  }
+  return t;
 }
 
 
@@ -644,180 +658,22 @@ static task_env *extract_first_delayed()
 
 
 static iotasks iot;
-#ifdef USE_SELECT
-static void iotasks_init(iotasks *iot)
-{
-	iot->maxfd = 0;
-	FD_ZERO(&iot->read_set);
-	FD_ZERO(&iot->write_set);
-	FD_ZERO(&iot->err_set);
-	link_init(&iot->tasks, type_hash("task_env"));
-}
-
-#if TASK_DBUG_ON
-static void poll_debug() MY_ATTRIBUTE((unused));
-static void poll_debug()
-{
-	GET_GOUT;
-#if 0
-	NDBG(FD_SETSIZE, d);
-	PTREXP(&iot.tasks);
-	NDBG(iot.tasks.type, u);
-	NDBG(cardinal(&iot.tasks), d);
-	PTREXP(iot.tasks.suc);
-	PTREXP(iot.tasks.pred);
-#endif
-	FWD_ITER(&iot.tasks, task_env,
-	    STRLIT("->");
-	    PTREXP(link_iter); PTREXP(link_iter->l.suc); PTREXP(link_iter->l.pred); NDBG(link_iter->waitfd, d);
-	    NDBG(FD_ISSET(link_iter->waitfd, &iot.read_set), d);
-	    NDBG(FD_ISSET(link_iter->waitfd, &iot.write_set), d);
-	    NDBG(FD_ISSET(link_iter->waitfd, &iot.err_set), d);
-	    );
-  PRINT_GOUT;
-  FREE_GOUT;
-}
-#endif
-
-
-static int	check_completion(task_env *t, fd_set *r, fd_set *w, fd_set *e)
-{
-	int	interrupt = 0;
-	assert(&t->l != &iot.tasks);
-	/* MAY_DBG(FN;
-		STREXP(t->name);
-		NDBG(t->waitfd,d);
-	    NDBG(FD_ISSET(t->waitfd,r),d);
-	    NDBG(FD_ISSET(t->waitfd,w),d);
-	    NDBG(FD_ISSET(t->waitfd,e),d);
-	); */
-	if (FD_ISSET(t->waitfd, e))
-		abort(); /* Close file here instead? */
-	interrupt = (t->time != 0.0 && t->time < task_now());
-	if (interrupt || /* timeout ? */
-	FD_ISSET(t->waitfd, r) || FD_ISSET(t->waitfd, w)) {
-		FD_CLR(t->waitfd, &iot.read_set);
-		FD_CLR(t->waitfd, &iot.write_set);
-		FD_CLR(t->waitfd, &iot.err_set);
-		t->interrupt = interrupt;
-		activate(t);
-		if (iot.maxfd - 1 == t->waitfd)
-			iot.maxfd = t->waitfd; /* Shrink set of watched files */
-		return 1;
-	} else {
-		return 0;
-	}
-}
-
-
-static int poll_wait(int ms)
-{
-	/* Wait at most ms milliseconds */
-        int wake= 0;
-	struct timeval select_timeout;
-	MAY_DBG(FN; NDBG(ms,d));
-	if (ms < 0 || ms > 1000)
-		ms = 1000; /* Wait at most 1000 ms */
-	/* convert milliseconds to seconds and microseconds */
-	select_timeout.tv_sec = ms / 1000;
-	select_timeout.tv_usec = (ms % 1000) * 1000;
-	 {
-		result	nfds = {0,0};
-		fd_set r = iot.read_set;
-		fd_set w = iot.write_set;
-		fd_set e = iot.err_set;
-		MAY_DBG(FN; poll_debug());
-		SET_OS_ERR(0);
-		while ((nfds.val = select(iot.maxfd, &r, &w, &e, &select_timeout)) == -1 ) {
-			nfds.funerr = to_errno(GET_OS_ERR);
-			if (hard_select_err(nfds.funerr)) {
-				task_dump_err(nfds.funerr);
-				DBGOUT(STRLIT("select failed"); NDBG(iot.maxfd, d));
-				return 0;
-			}
-			SET_OS_ERR(0);
-			r = iot.read_set;
-			w = iot.write_set;
-			e = iot.err_set;
-		}
-		/* MAY_DBG(FN; poll_debug()); */
-		/* Wake up ready tasks */
-		/* if (nfds.val > 0) { */
-		/*   FWD_ITER(&iot.tasks, task_env,  */
-		/*            nfds.val -= check_completion(link_iter, &r, &w, &e);  */
-		/*            if (nfds.val == 0) break); */
-		/* } */
-		FWD_ITER(&iot.tasks, task_env,
-		    if(check_completion(link_iter, &r, &w, &e)) wake=1);
-	}
-        return wake;
-}
-
-
-static void add_fd(task_env *t, int fd, int op)
-{
-	MAY_DBG(FN; PTREXP(t); STREXP(t->name); NDBG(fd, d); NDBG(op, c));
-	assert(fd >= 0);
-	t->waitfd = fd;
-	if (fd >= iot.maxfd)
-		iot.maxfd = fd + 1;
-	FD_CLR(fd, &iot.err_set);
-	if ('r' == op)
-		FD_SET(fd, &iot.read_set);
-	else
-		FD_SET(fd, &iot.write_set);
-	task_wait(t, &iot.tasks);
-	/* MAY_DBG(FN; poll_debug()); */
-}
-
-
-static void unpoll(int i)
-{
-	FD_CLR(i, &iot.read_set);
-	FD_CLR(i, &iot.write_set);
-	FD_CLR(i, &iot.err_set);
-}
-
-
-static void wake_all_io()
-{
-	FWD_ITER(&iot.tasks, task_env,
-	    unpoll(link_iter->waitfd);
-	    activate(link_iter);
-	    );
-}
-
-
-void remove_and_wakeup(int fd)
-{
-	MAY_DBG(FN; NDBG(fd, d));
-	FWD_ITER(&iot.tasks, task_env,
-	    if (fd == link_iter->waitfd) {
-		unpoll(link_iter->waitfd);
-		    activate(link_iter);
-		    if (iot.maxfd - 1 == link_iter->waitfd)
-		    iot.maxfd = link_iter->waitfd; /* Shrink set of watched files */
-	}
-	);
-}
-
-
-#else
-static int	active_io()
-{
-	return iot.nwait > 0;
-}
-
 
 static void iotasks_init(iotasks *iot)
-{
-	int	i;
-	iot->nwait = 0;
-	for (i = 0; i < MAXFILES; i++) {
-		iot->tasks[i] = 0;
-	}
-}
+ {
+  DBGOUT(FN);
+  iot->nwait = 0;
+  init_pollfd_array(&iot->fd);
+  init_task_env_p_array(&iot->tasks);
+ }
 
+static void iotasks_deinit(iotasks *iot)
+{
+  DBGOUT(FN);
+  iot->nwait = 0;
+  free_pollfd_array(&iot->fd);
+  free_task_env_p_array(&iot->tasks);
+}
 
 #if TASK_DBUG_ON
 static void poll_debug()
@@ -831,145 +687,104 @@ static void poll_debug()
 
 static void poll_wakeup(int i)
 {
-	activate(task_unref(iot.tasks[i]));
-	iot.tasks[i] = NULL;
-	iot.nwait--; /* Shrink array of pollfds */
-	iot.fd[i] = iot.fd[iot.nwait];
-	iot.tasks[i] = iot.tasks[iot.nwait];
+  activate(task_unref(get_task_env_p(&iot.tasks,i)));
+  set_task_env_p(&iot.tasks, NULL,i);
+  iot.nwait--; /* Shrink array of pollfds */
+  set_pollfd(&iot.fd, get_pollfd(&iot.fd,iot.nwait),i);
+  set_task_env_p(&iot.tasks, get_task_env_p(&iot.tasks,iot.nwait),i);
 }
 
+static int poll_wait(int ms) {
+  result nfds = {0, 0};
+  int wake = 0;
 
-static int poll_wait(int ms)
-{
-	result	nfds = {0,0};
-
-	/* Wait at most ms milliseconds */
-	MAY_DBG(FN; NDBG(ms,d));
-        int wake= 0;
-	if (ms < 0 || ms > 1000)
-		ms = 1000; /* Wait at most 1000 ms */
-	SET_OS_ERR(0);
-	while ((nfds.val = poll (iot.fd, iot.nwait, ms)) == -1 ) {
-		nfds.err = to_errno(GET_OS_ERR);
-		if (nfds.err != SOCK_EINTR ) {
-			task_dump_err(nfds.err);
-			MAY_DBG(FN; STRLIT("poll failed"));
-			abort();
-		}
-		SET_OS_ERR(0);
-	}
-	/* Wake up ready tasks */
-	 {
-		int	i = 0;
-		int	interrupt = 0;
-		while (i < iot.nwait) {
-			interrupt = (iot.tasks[i]->time != 0.0 && iot.tasks[i]->time < task_now());
-			if (interrupt || /* timeout ? */
-			iot.fd[i].revents) {
-				/* if(iot.fd[i].revents & POLLERR) abort(); */
-				iot.tasks[i]->interrupt = interrupt;
-				poll_wakeup(i);
-                                wake= 1;
-			} else {
-				i++;
-			}
-		}
-	}
-        return wake;
+  /* Wait at most ms milliseconds */
+  MAY_DBG(FN; NDBG(ms, d));
+  if (ms < 0 || ms > 1000) ms = 1000; /* Wait at most 1000 ms */
+  SET_OS_ERR(0);
+  while ((nfds.val = poll(iot.fd.pollfd_array_val, iot.nwait, ms)) == -1) {
+    nfds.funerr = to_errno(GET_OS_ERR);
+    if (nfds.funerr != SOCK_EINTR) {
+      task_dump_err(nfds.funerr);
+      MAY_DBG(FN; STRLIT("poll failed"));
+      abort();
+    }
+    SET_OS_ERR(0);
+  }
+  /* Wake up ready tasks */
+  {
+    int i = 0;
+    int interrupt = 0;
+    while (i < iot.nwait) {
+      interrupt =
+        (get_task_env_p(&iot.tasks,i)->time != 0.0 &&
+        get_task_env_p(&iot.tasks,i)->time < task_now());
+      if (interrupt || /* timeout ? */
+        get_pollfd(&iot.fd,i).revents) {
+        /* if(iot.fd[i].revents & POLLERR) abort(); */
+        get_task_env_p(&iot.tasks,i)->interrupt = interrupt;
+        poll_wakeup(i);
+        wake = 1;
+      } else {
+        i++;
+      }
+    }
+  }
+  return wake;
 }
 
-
-static void add_fd(task_env *t, int fd, int op)
-{
-	int	events = 'r' == op ? POLLIN | POLLRDNORM : POLLOUT;
-	MAY_DBG(FN; PTREXP(t); NDBG(fd,d); NDBG(op,d));
-	assert(fd >= 0);
-	assert(fd < MAXFILES);
-	t->waitfd = fd;
-	deactivate(t);
-	task_ref(t);
-	iot.tasks[iot.nwait] = t;
-	iot.fd[iot.nwait].fd = fd;
-	iot.fd[iot.nwait].events = events;
-	iot.fd[iot.nwait].revents = 0;
-	iot.nwait++;
+static void add_fd(task_env *t, int fd, int op) {
+  int events = 'r' == op ? POLLIN | POLLRDNORM : POLLOUT;
+  MAY_DBG(FN; PTREXP(t); NDBG(fd, d); NDBG(op, d));
+  assert(fd >= 0);
+  t->waitfd = fd;
+  deactivate(t);
+  task_ref(t);
+  set_task_env_p(&iot.tasks, t, iot.nwait);
+  {
+    pollfd x;
+    x.fd = fd;
+    x.events = events;
+    x.revents = 0;
+    set_pollfd(&iot.fd, x, iot.nwait);
+  }
+  iot.nwait++;
 }
 
-
-void unpoll(int i)
-{
-	assert(i < MAXFILES);
-	iot.tasks[i] = NULL;
-	iot.fd[i].fd = -1;
+void unpoll(int i) {
+  task_unref(get_task_env_p(&iot.tasks, i));
+  set_task_env_p(&iot.tasks, NULL,i);
+  {
+    pollfd x;
+    x.fd = -1;
+    x.events = 0;
+    x.revents = 0;
+    set_pollfd(&iot.fd, x, i);
+  }
 }
 
-
-static void wake_all_io()
-{
-	int	i;
-	for (i = 0; i < iot.nwait; i++) {
-		unpoll(i);
-		activate(task_unref(iot.tasks[i]));
-	}
-	iot.nwait = 0;
+static void wake_all_io() {
+  int i;
+  for (i = 0; i < iot.nwait; i++) {
+    activate(get_task_env_p(&iot.tasks,i));
+    unpoll(i);
+  }
+  iot.nwait = 0;
 }
 
-
-void remove_and_wakeup(int fd)
-{
-	int	i = 0;
-	MAY_DBG(FN; NDBG(fd,d));
-	while (i < iot.nwait) {
-		if (iot.fd[i].fd == fd) {
-			poll_wakeup(i);
-		} else {
-			i++;
-		}
-	}
+void remove_and_wakeup(int fd) {
+  int i = 0;
+  MAY_DBG(FN; NDBG(fd, d));
+  while (i < iot.nwait) {
+    if (get_pollfd(&iot.fd,i).fd == fd) {
+      poll_wakeup(i);
+    } else {
+      i++;
+    }
+  }
 }
 
-
-#endif
 task_env *stack = NULL;
-/* Locks needed to get atomic reads and writes by protecting file descriptors
-		   while the task has been suspended by wait_io */
-static task_env *io_wait_locks[MAXFILES][2];
-/* purecov: begin deadcode */
-int	is_locked(int fd)
-{
-	return io_wait_locks[fd][0] || io_wait_locks[fd][1];
-}
-
-
-int	lock_fd(int fd, task_env *t, int lock)
-{
-	if (fd < 0)
-		return 0;
-	lock = lock != 'r';
-	if (io_wait_locks[fd][lock]) {
-		DBGOUT(FN; NDBG(fd, d); PTREXP(t); STRLIT(" failed"));
-		return 0;
-	} else {
-		io_wait_locks[fd][lock] = t;
-		return 1;
-	}
-}
-
-
-int	unlock_fd(int fd, task_env *t, int lock)
-{
-	if (fd < 0)
-		return 0;
-	lock = lock != 'r';
-	if (io_wait_locks[fd][lock] != t) {
-		DBGOUT(FN; NDBG(fd, d); PTREXP(t); STRLIT(" failed"));
-		return 0;
-	} else {
-		io_wait_locks[fd][lock] = NULL;
-		return 1;
-	}
-}
-/* purecov: end */
 
 task_env *wait_io(task_env *t, int fd, int op)
 {
@@ -1304,6 +1119,7 @@ done_wait:
 			idle_time += seconds() - time;
 		}
 	}
+  task_sys_deinit();
 }
 
 
@@ -1506,7 +1322,7 @@ result	announce_tcp(xcom_port port)
                     "0.0.0.0", port, fd.val, err);
 		goto err;
 	}
-        G_MESSAGE("Successfully bound to %s:%d (socket=%d).",
+        G_DEBUG("Successfully bound to %s:%d (socket=%d).",
                   "0.0.0.0", port, fd.val);
 	if (listen(fd.val, 32) < 0) {
           int err = to_errno(GET_OS_ERR);
@@ -1514,7 +1330,7 @@ result	announce_tcp(xcom_port port)
                     "(socket=%d, errno=%d)!", fd.val, err);
 		goto err;
 	}
-        G_MESSAGE("Successfully set listen backlog to 32 "
+        G_DEBUG("Successfully set listen backlog to 32 "
                   "(socket=%d)!", fd.val);
 	/* Make socket non-blocking */
 	unblock_fd(fd.val);
@@ -1525,8 +1341,9 @@ result	announce_tcp(xcom_port port)
                     fd.val, err);
         }
         else
-          G_MESSAGE("Successfully unblocked socket (socket=%d)!",
-                    fd.val);
+        {
+          G_DEBUG("Successfully unblocked socket (socket=%d)!", fd.val);
+        }
 	return fd;
 
  err:
@@ -1631,6 +1448,13 @@ void task_sys_init()
 	/* task_new(statistics_task, null_arg, "statistics_task", 1);  */
 }
 
+
+static void task_sys_deinit()
+{
+  DBGOUT(FN);
+  iotasks_deinit(&iot);
+}
+
 /* purecov: begin deadcode */
 int	is_running(task_env *t)
 {
