@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -34,6 +34,7 @@
 #include "xcom_detector.h"
 #include "site_struct.h"
 #include "node_connection.h"
+#include "node_list.h"
 #include "xcom_transport.h"
 #include "xcom_statistics.h"
 #include "xcom_base.h"
@@ -417,7 +418,11 @@ x_putbytes (XDR *xdrs, const char *bp MY_ATTRIBUTE((unused)), u_int len)
 
 
 static u_int
+#if defined(__APPLE__) || defined(__FreeBSD__) || defined(HAVE_TIRPC)
+x_getpostn(XDR *xdrs)
+#else
 x_getpostn (const XDR *xdrs)
+#endif
 {
 #ifdef OLD_XDR
   return (u_int)(xdrs->x_handy);
@@ -657,7 +662,7 @@ int serialize_msg(pax_msg *p, xcom_proto x_proto, uint32_t *buflen, char **buf)
 	*buf = 0;
 
 	return old_proto_knows(x_proto, p->op) &&
-		serialize((void * )p, x_proto, buflen, (xdrproc_t)xdr_pax_msg, buf);
+						serialize((void *)p, x_proto, buflen, (xdrproc_t)xdr_pax_msg, buf);
 }
 
 int deserialize_msg(pax_msg *p, xcom_proto x_proto,  char *buf, uint32_t buflen)
@@ -724,6 +729,7 @@ mksrv(char *srv, xcom_port port)
 		abort();
 	}
 	s->garbage = 0;
+	s->invalid = 0;
 	s->refcnt = 0;
 	s->srv = srv;
 	s->port = port;
@@ -741,6 +747,12 @@ mksrv(char *srv, xcom_port port)
 		s->reply_handler = task_new(reply_handler_task, void_arg(s), "reply_handler_task", XCOM_THREAD_DEBUG);
 	}
 	reset_srv_buf(&s->out_buf);
+	/*
+	 Keep the server from being freed if the acceptor_learner_task calls
+	 srv_unref on the server before the {local_,}server_task and
+	 reply_handler_task begin.
+	*/
+	srv_ref(s);
 	return s;
 }
 
@@ -866,6 +878,9 @@ static void shut_srv(server *s)
 		task_terminate(s->sender);
 	if (s->reply_handler)
 		task_terminate(s->reply_handler);
+
+	// Allow the server to be freed. This unref pairs with the ref from mksrv.
+	srv_unref(s);
 }
 
 
@@ -903,19 +918,20 @@ int tcp_server(task_arg arg) {
   ep->refused = 0;
   unblock_fd(ep->fd);
   DBGOUT(FN; NDBG(ep->fd, d););
+  G_MESSAGE("XCom protocol version: %d", my_xcom_version);
   G_MESSAGE(
-      "Ready to accept incoming connections on %s:%d "
-      "(socket=%d)!",
-      "0.0.0.0", xcom_listen_port, ep->fd);
+      "XCom initialized and ready to accept incoming connections on port %d",
+      xcom_listen_port);
   do {
     TASK_CALL(accept_tcp(ep->fd, &ep->cfd));
     /* Callback to check that the file descriptor is accepted. */
-    if (xcom_socket_accept_callback && !xcom_socket_accept_callback(ep->cfd)) {
+    if (xcom_socket_accept_callback &&
+        !xcom_socket_accept_callback(ep->cfd, get_site_def())) {
       shut_close_socket(&ep->cfd);
       ep->cfd = -1;
     }
     if(ep->cfd == -1){
-      G_MESSAGE("accept failed");
+      G_DEBUG("accept failed");
       ep->refused = 1;
       TASK_DELAY(0.1);
     } else {
@@ -1040,7 +1056,7 @@ static inline int	_send_server_msg(site_def const *s, node_no to, pax_msg *p)
 {
 	assert(s);
 	assert(s->servers[to]);
-	if (s->servers[to] && p) {
+	if (s->servers[to] && s->servers[to]->invalid == 0 && p) {
 		send_msg(s->servers[to], s->nodeno, to, get_group_id(s), p);
 	}
 	return 0;
@@ -1779,7 +1795,7 @@ static xcom_port get_port(char *a)
 
 xcom_port xcom_get_port(char *a)
 {
-	return get_port(a);
+	return a ? get_port(a): 0;
 }
 
 
@@ -1795,7 +1811,7 @@ static server *find_server(server *table[], int n, char *name, xcom_port port)
 }
 
 
-void update_servers(site_def *s)
+void update_servers(site_def *s, cargo_type operation)
 {
 	u_int	n;
 
@@ -1815,6 +1831,8 @@ void update_servers(site_def *s)
 				DBGOUT(FN; STRLIT("re-using server "); NDBG(i, d); STREXP(name));
 				free(name);
 				s->servers[i] = sp;
+				if(sp->invalid)
+					sp->invalid= 0;
 			} else { /* No server? Create one */
 				DBGOUT(FN; STRLIT("creating new server "); NDBG(i, d); STREXP(name));
 				if (port > 0)
@@ -1827,6 +1845,40 @@ void update_servers(site_def *s)
 		for (i = n; i < NSERVERS; i++) {
 			s->servers[i] = 0;
 		}
+		/*
+		If we have a force config, mark the servers that do not belong to this
+		configuration as invalid
+		*/
+
+		if(operation == force_config_type) {
+			const site_def* old_site_def= get_prev_site_def();
+			invalidate_servers(old_site_def, s);
+		}
+
+	}
+}
+
+
+/*
+ Make a diff between 2 site_defs and mark as invalid servers
+ that do not belong to the new site_def.
+ This is only to be used if we are forcing a configuration.
+*/
+void invalidate_servers(const site_def* old_site_def,
+			const site_def* new_site_def) {
+	u_int node= 0;
+	for(; node < get_maxnodes(old_site_def); node++){
+		node_address* node_addr_from_old_site_def= &old_site_def->nodes.node_list_val[node];
+		if(!node_exists(node_addr_from_old_site_def, &new_site_def->nodes))
+		{
+			char *addr = node_addr_from_old_site_def->address;
+			char *name = get_name(addr);
+			xcom_port port = get_port(addr);
+			server *sp = find_server(all_servers, maxservers, name, port);
+			if (sp) {
+				sp->invalid= 1;
+			}
+		}
 	}
 }
 
