@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -466,7 +466,8 @@ internal_wait_for_server_gtid_executed(Sql_service_interface *sql_interface,
   }
 
   std::string query= ss.str();
-  long srv_err= sql_interface->execute_query(query);
+  Sql_resultset rset;
+  long srv_err= sql_interface->execute_query(query, &rset);
   if (srv_err)
   {
     /* purecov: begin inspected */
@@ -478,6 +479,11 @@ internal_wait_for_server_gtid_executed(Sql_service_interface *sql_interface,
     DBUG_RETURN(1);
     /* purecov: end */
   }
+  else if(rset.get_rows() > 0)
+  {
+    if (rset.getLong(0) == 1)
+      DBUG_RETURN(-1);
+  }
   DBUG_RETURN(0);
 }
 
