@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -263,8 +263,8 @@ ACL_PROXY_USER::check_validity(bool check_no_resolve)
 {
   if (check_no_resolve && 
       (hostname_requires_resolving(host.get_host()) ||
-       hostname_requires_resolving(proxied_host.get_host())))
-  {
+       hostname_requires_resolving(proxied_host.get_host())) &&
+      strcmp(host.get_host(), "localhost") != 0) {
     sql_print_warning("'proxies_priv' entry '%s@%s %s@%s' "
                       "ignored in --skip-name-resolve mode.",
                       proxied_user ? proxied_user : "",
@@ -730,7 +730,7 @@ bool GRANT_TABLE::init(TABLE *col_privs)
 
     if (!col_privs->key_info)
     {
-      my_error(ER_TABLE_CORRUPT, MYF(0), col_privs->s->db.str,
+      my_error(ER_MISSING_KEY, MYF(0), col_privs->s->db.str,
                col_privs->s->table_name.str);
       return true;
     }
@@ -1495,8 +1495,18 @@ static my_bool acl_load(THD *thd, TABLE_LIST *tables)
    We need to check whether we are working with old database layout. This
    might be the case for instance when we are running mysql_upgrade.
   */
-  table_schema= user_table_schema_factory.get_user_table_schema(table);
-  is_old_db_layout= user_table_schema_factory.is_old_user_table_schema(table);
+  if (user_table_schema_factory.user_table_schema_check(table))
+  {
+    table_schema= user_table_schema_factory.get_user_table_schema(table);
+    is_old_db_layout= user_table_schema_factory.is_old_user_table_schema(table);
+  }
+  else
+  {
+    sql_print_error("[FATAL] mysql.user table is damaged. "
+                    "Please run mysql_upgrade.");
+    end_read_record(&read_record_info);
+    goto end;
+  }
 
   allow_all_hosts=0;
   int read_rec_errcode;
@@ -1525,8 +1535,8 @@ static my_bool acl_load(THD *thd, TABLE_LIST *tables)
                                       table->field[table_schema->host_idx()]));
     user.user= get_field(&global_acl_memory,
                          table->field[table_schema->user_idx()]);
-    if (check_no_resolve && hostname_requires_resolving(user.host.get_host()))
-    {
+  if (check_no_resolve && hostname_requires_resolving(user.host.get_host()) &&
+      strcmp(user.host.get_host(), "localhost") != 0) {
       sql_print_warning("'user' entry '%s@%s' "
                         "ignored in --skip-name-resolve mode.",
                         user.user ? user.user : "",
@@ -1847,8 +1857,15 @@ static my_bool acl_load(THD *thd, TABLE_LIST *tables)
   } // END while reading records from the mysql.user table
 
   end_read_record(&read_record_info);
+
+  DBUG_EXECUTE_IF("simulate_acl_init_failure",
+                  read_rec_errcode= HA_ERR_WRONG_IN_RECORD;);
+
   if (read_rec_errcode > 0)
+  {
+    table->file->print_error(read_rec_errcode, MYF(ME_ERRORLOG));
     goto end;
+  }
 
   std::sort(acl_users->begin(), acl_users->end(), ACL_compare());
   acl_users->shrink_to_fit();
@@ -1893,8 +1910,8 @@ static my_bool acl_load(THD *thd, TABLE_LIST *tables)
       continue;
     }
     db.user=get_field(&global_acl_memory, table->field[MYSQL_DB_FIELD_USER]);
-    if (check_no_resolve && hostname_requires_resolving(db.host.get_host()))
-    {
+    if (check_no_resolve && hostname_requires_resolving(db.host.get_host()) &&
+        strcmp(db.host.get_host(), "localhost") != 0) {
       sql_print_warning("'db' entry '%s %s@%s' "
                         "ignored in --skip-name-resolve mode.",
                         db.db,
@@ -1933,7 +1950,10 @@ static my_bool acl_load(THD *thd, TABLE_LIST *tables)
 
   end_read_record(&read_record_info);
   if (read_rec_errcode > 0)
+  {
+    table->file->print_error(read_rec_errcode, MYF(ME_ERRORLOG));
     goto end;
+  }
 
   std::sort(acl_dbs->begin(), acl_dbs->end(), ACL_compare());
   acl_dbs->shrink_to_fit();
@@ -1963,7 +1983,10 @@ static my_bool acl_load(THD *thd, TABLE_LIST *tables)
 
     end_read_record(&read_record_info);
     if (read_rec_errcode > 0)
+    {
+      table->file->print_error(read_rec_errcode, MYF(ME_ERRORLOG));
       goto end;
+    }
 
     std::sort(acl_proxy_users->begin(), acl_proxy_users->end(), ACL_compare());
   }
@@ -2432,8 +2455,8 @@ static my_bool grant_load(THD *thd, TABLE_LIST *tables)
 
       if (check_no_resolve)
       {
-        if (hostname_requires_resolving(mem_check->host.get_host()))
-        {
+        if (hostname_requires_resolving(mem_check->host.get_host()) &&
+            strcmp(mem_check->host.get_host(), "localhost") != 0) {
           sql_print_warning("'tables_priv' entry '%s %s@%s' "
                             "ignored in --skip-name-resolve mode.",
                             mem_check->tname,
