@@ -1,33 +1,27 @@
-/* Copyright (c) 2002, 2011, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2002, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License, version 2.0,
-   as published by the Free Software Foundation.
-
-   This program is also distributed with certain software (including
-   but not limited to OpenSSL) that is licensed under separate terms,
-   as designated in a particular file or component or in included license
-   documentation.  The authors of MySQL hereby grant you an additional
-   permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License, version 2.0, for more details.
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
    51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA */
 
-#include "sql_priv.h"
-#include "unireg.h"
 #include "sql_help.h"
 #include "sql_table.h"                          // primary_key_name
-#include "sql_base.h"               // REPORT_ALL_ERRORS, setup_tables
+#include "sql_base.h"               // REPORT_ALL_ERRORS
 #include "opt_range.h"              // SQL_SELECT
 #include "opt_trace.h"              // Opt_trace_object
 #include "records.h"          // init_read_record, end_read_record
+#include "debug_sync.h"
+#include "sql_executor.h"                       // QEP_TAB
+#include "item_cmpfunc.h"           // Item_func_like
 
 struct st_find_field
 {
@@ -94,7 +88,7 @@ enum enum_used_fields
 static bool init_fields(THD *thd, TABLE_LIST *tables,
 			struct st_find_field *find_fields, uint count)
 {
-  Name_resolution_context *context= &thd->lex->select_lex.context;
+  Name_resolution_context *context= &thd->lex->select_lex->context;
   DBUG_ENTER("init_fields");
   context->resolve_in_table_list_only(tables);
   for (; count-- ; find_fields++)
@@ -104,8 +98,9 @@ static bool init_fields(THD *thd, TABLE_LIST *tables,
                                       "mysql", find_fields->table_name,
                                       find_fields->field_name);
     if (!(find_fields->field= find_field_in_tables(thd, field, tables, NULL,
-						   0, REPORT_ALL_ERRORS, 1,
-                                                   TRUE)))
+						   0, REPORT_ALL_ERRORS,
+                                                   false, // No priv checking
+                                                   true)))
       DBUG_RETURN(1);
     bitmap_set_bit(find_fields->field->table->read_set,
                    find_fields->field->field_index);
@@ -192,22 +187,23 @@ void memorize_variant_topic(THD *thd, TABLE *topics, int count,
 
 */
 
-int search_topics(THD *thd, TABLE *topics, struct st_find_field *find_fields,
-		  SQL_SELECT *select, List<String> *names,
+int search_topics(THD *thd, QEP_TAB *topics, struct st_find_field *find_fields,
+		  List<String> *names,
 		  String *name, String *description, String *example)
 {
   int count= 0;
   READ_RECORD read_record_info;
   DBUG_ENTER("search_topics");
 
-  if (init_read_record(&read_record_info, thd, topics, select, 1, 0, FALSE))
+  if (init_read_record(&read_record_info, thd, NULL, topics,
+                       1, 0, FALSE))
     DBUG_RETURN(0);
 
   while (!read_record_info.read_record(&read_record_info))
   {
-    if (!select->cond->val_int())		// Doesn't match like
+    if (!topics->condition()->val_int())        // Doesn't match like
       continue;
-    memorize_variant_topic(thd,topics,count,find_fields,
+    memorize_variant_topic(thd,topics->table(),count,find_fields,
 			   names,name,description,example);
     count++;
   }
@@ -235,19 +231,19 @@ int search_topics(THD *thd, TABLE *topics, struct st_find_field *find_fields,
     2   found more then one topic matching the mask
 */
 
-int search_keyword(THD *thd, TABLE *keywords, struct st_find_field *find_fields,
-                   SQL_SELECT *select, int *key_id)
+int search_keyword(THD *thd, QEP_TAB *keywords, struct st_find_field *find_fields,
+                   int *key_id)
 {
   int count= 0;
   READ_RECORD read_record_info;
   DBUG_ENTER("search_keyword");
 
-  if (init_read_record(&read_record_info, thd, keywords, select, 1, 0, FALSE))
+  if (init_read_record(&read_record_info, thd, NULL, keywords, 1, 0, FALSE))
     DBUG_RETURN(0);
 
   while (!read_record_info.read_record(&read_record_info) && count<2)
   {
-    if (!select->cond->val_int())		// Dosn't match like
+    if (!keywords->condition()->val_int())		// Dosn't match like
       continue;
 
     *key_id= (int)find_fields[help_keyword_help_keyword_id].field->val_int();
@@ -331,7 +327,7 @@ int get_topics_for_keyword(THD *thd, TABLE *topics, TABLE *relations,
     uchar topic_id_buff[8];
     longlong topic_id= rtopic_id->val_int();
     Field *field= find_fields[help_topic_help_topic_id].field;
-    field->store((longlong) topic_id, TRUE);
+    field->store(topic_id, TRUE);
     field->get_key_image(topic_id_buff, field->pack_length(), Field::itRAW);
 
     if (!topics->file->ha_index_read_map(topics->record[0], topic_id_buff,
@@ -365,9 +361,9 @@ int get_topics_for_keyword(THD *thd, TABLE *topics, TABLE *relations,
     #			Number of categories found
 */
 
-int search_categories(THD *thd, TABLE *categories,
+int search_categories(THD *thd, QEP_TAB *categories,
 		      struct st_find_field *find_fields,
-		      SQL_SELECT *select, List<String> *names, int16 *res_id)
+		      List<String> *names, int16 *res_id)
 {
   Field *pfname= find_fields[help_category_name].field;
   Field *pcat_id= find_fields[help_category_help_category_id].field;
@@ -376,13 +372,13 @@ int search_categories(THD *thd, TABLE *categories,
 
   DBUG_ENTER("search_categories");
 
-  if (init_read_record(&read_record_info, thd, categories, select,
+  if (init_read_record(&read_record_info, thd, NULL, categories,
                        1, 0, FALSE))
     DBUG_RETURN(0);
     
   while (!read_record_info.read_record(&read_record_info))
   {
-    if (select && !select->cond->val_int())
+    if (categories->condition() && !categories->condition()->val_int())
       continue;
     String *lname= new (thd->mem_root) String;
     get_field(thd->mem_root,pfname,lname);
@@ -407,18 +403,18 @@ int search_categories(THD *thd, TABLE *categories,
     res     list of finded names
 */
 
-void get_all_items_for_category(THD *thd, TABLE *items, Field *pfname,
-				SQL_SELECT *select, List<String> *res)
+void get_all_items_for_category(THD *thd, QEP_TAB *items, Field *pfname,
+				List<String> *res)
 {
   READ_RECORD read_record_info;
   DBUG_ENTER("get_all_items_for_category");
 
-  if (init_read_record(&read_record_info, thd, items, select,
+  if (init_read_record(&read_record_info, thd, NULL, items,
                        1, 0, FALSE))
     DBUG_VOID_RETURN;
   while (!read_record_info.read_record(&read_record_info))
   {
-    if (!select->cond->val_int())
+    if (!items->condition()->val_int())
       continue;
     String *name= new (thd->mem_root) String();
     get_field(thd->mem_root,pfname,name);
@@ -434,6 +430,7 @@ void get_all_items_for_category(THD *thd, TABLE *items, Field *pfname,
 
   SYNOPSIS
     send_answer_1()
+    thd      - THD to get the current protocol from and call send_result_metadata
     protocol - protocol for sending
     s1 - value of column "Name"
     s2 - value of column "Description"
@@ -454,7 +451,7 @@ void get_all_items_for_category(THD *thd, TABLE *items, Field *pfname,
     0		Successeful send
 */
 
-int send_answer_1(Protocol *protocol, String *s1, String *s2, String *s3)
+int send_answer_1(THD *thd, String *s1, String *s2, String *s3)
 {
   DBUG_ENTER("send_answer_1");
   List<Item> field_list;
@@ -462,15 +459,15 @@ int send_answer_1(Protocol *protocol, String *s1, String *s2, String *s3)
   field_list.push_back(new Item_empty_string("description",1000));
   field_list.push_back(new Item_empty_string("example",1000));
 
-  if (protocol->send_result_set_metadata(&field_list,
-                            Protocol::SEND_NUM_ROWS | Protocol::SEND_EOF))
+  if (thd->send_result_metadata(&field_list,
+                                Protocol::SEND_NUM_ROWS | Protocol::SEND_EOF))
     DBUG_RETURN(1);
 
-  protocol->prepare_for_resend();
-  protocol->store(s1);
-  protocol->store(s2);
-  protocol->store(s3);
-  if (protocol->write())
+  thd->get_protocol()->start_row();
+  thd->get_protocol()->store(s1);
+  thd->get_protocol()->store(s2);
+  thd->get_protocol()->store(s3);
+  if (thd->get_protocol()->end_row())
     DBUG_RETURN(-1);
   DBUG_RETURN(0);
 }
@@ -481,7 +478,7 @@ int send_answer_1(Protocol *protocol, String *s1, String *s2, String *s3)
 
   SYNOPSIS
    send_header_2()
-    protocol       - protocol for sending
+    thd            - thread to get the current status from
     is_it_category - need column 'source_category_name'
 
   IMPLEMENTATION
@@ -497,7 +494,7 @@ int send_answer_1(Protocol *protocol, String *s1, String *s2, String *s3)
     result of protocol->send_result_set_metadata
 */
 
-int send_header_2(Protocol *protocol, bool for_category)
+int send_header_2(THD *thd, bool for_category)
 {
   DBUG_ENTER("send_header_2");
   List<Item> field_list;
@@ -505,8 +502,8 @@ int send_header_2(Protocol *protocol, bool for_category)
     field_list.push_back(new Item_empty_string("source_category_name",64));
   field_list.push_back(new Item_empty_string("name",64));
   field_list.push_back(new Item_empty_string("is_it_category",1));
-  DBUG_RETURN(protocol->send_result_set_metadata(&field_list, Protocol::SEND_NUM_ROWS |
-                                                 Protocol::SEND_EOF));
+  DBUG_RETURN(thd->send_result_metadata(&field_list,
+    Protocol::SEND_NUM_ROWS | Protocol::SEND_EOF));
 }
 
 /*
@@ -563,84 +560,87 @@ int send_variant_2_list(MEM_ROOT *mem_root, Protocol *protocol,
 
   for (pos= pointers; pos!=end; pos++)
   {
-    protocol->prepare_for_resend();
+    protocol->start_row();
     if (source_name)
       protocol->store(source_name);
     protocol->store(*pos);
     protocol->store(cat,1,&my_charset_latin1);
-    if (protocol->write())
+    if (protocol->end_row())
       DBUG_RETURN(-1);
   }
 
   DBUG_RETURN(0);
 }
 
-/*
-  Prepare simple SQL_SELECT table.* WHERE <Item>
+/**
+  Prepare access method to do "SELECT * FROM table WHERE <cond>"
 
-  SYNOPSIS
-    prepare_simple_select()
-    thd      Thread handler
-    cond     WHERE part of select
-    table    goal table
+  @param thd      Thread handler
+  @param cond     WHERE part of select
+  @param table    goal table
+  @param tab      QEP_TAB
 
-    error    code of error (out)
+  @returns true if error
 
-  RETURN VALUES
-    #  created SQL_SELECT
+  @note Side-effects: 'table', 'cond' and possibly a 'quick' are assigned to
+  'tab'
 */
 
-SQL_SELECT *prepare_simple_select(THD *thd, Item *cond,
-				  TABLE *table, int *error)
+bool prepare_simple_select(THD *thd, Item *cond,
+                           TABLE *table, QEP_TAB *tab)
 {
   if (!cond->fixed)
     cond->fix_fields(thd, &cond);	// can never fail
 
+  // Initialize the cost model that will be used for this table
+  table->init_cost_model(thd->cost_model());
+
   /* Assume that no indexes cover all required fields */
   table->covering_keys.clear_all();
 
-  SQL_SELECT *res= make_select(table, 0, 0, cond, 0, error);
+  tab->set_table(table);
+  tab->set_condition(cond);
 
   // Wrapper for correct JSON in optimizer trace
   Opt_trace_object wrapper(&thd->opt_trace);
-  if (*error || (res && res->check_quick(thd, 0, HA_POS_ERROR)) ||
-      (res && res->quick && res->quick->reset()))
-  {
-    delete res;
-    res=0;
-  }
-  return res;
+  key_map keys_to_use(key_map::ALL_BITS), needed_reg_dummy;
+  QUICK_SELECT_I *qck;
+  const bool impossible=
+    test_quick_select(thd, keys_to_use, 0, HA_POS_ERROR, false,
+                      ORDER::ORDER_NOT_RELEVANT, tab, cond,
+                      &needed_reg_dummy, &qck, tab->table()->force_index) < 0;
+  tab->set_quick(qck);
+
+  return impossible || (tab->quick() && tab->quick()->reset());
 }
 
-/*
-  Prepare simple SQL_SELECT table.* WHERE table.name LIKE mask
 
-  SYNOPSIS
-    prepare_select_for_name()
-    thd      Thread handler
-    mask     mask for compare with name
-    mlen     length of mask
-    tables   list of tables, used in WHERE
-    table    goal table
-    pfname   field "name" in table
+/**
+  Prepare access method to do "SELECT * FROM table LIKE mask"
 
-    error    code of error (out)
+  @param  thd      Thread handler
+  @param  mask     mask for compare with name
+  @param  mlen     length of mask
+  @param  tables   list of tables, used in WHERE
+  @param  table    goal table
+  @param  pfname   field "name" in table
+  @param  tab      QEP_TAB
 
-  RETURN VALUES
-    #  created SQL_SELECT
+  @returns true if error
+  @see prepare_simple_select()
 */
 
-SQL_SELECT *prepare_select_for_name(THD *thd, const char *mask, uint mlen,
-				    TABLE_LIST *tables, TABLE *table,
-				    Field *pfname, int *error)
+bool prepare_select_for_name(THD *thd, const char *mask, size_t mlen,
+                             TABLE_LIST *tables, TABLE *table,
+                             Field *pfname, QEP_TAB *tab)
 {
   Item *cond= new Item_func_like(new Item_field(pfname),
 				 new Item_string(mask,mlen,pfname->charset()),
 				 new Item_string("\\",1,&my_charset_latin1),
                                  FALSE);
   if (thd->is_fatal_error)
-    return 0;					// OOM
-  return prepare_simple_select(thd, cond, table, error);
+    return true;                                /* purecov: inspected */
+  return prepare_simple_select(thd, cond, table, tab);
 }
 
 
@@ -658,17 +658,16 @@ SQL_SELECT *prepare_select_for_name(THD *thd, const char *mask, uint mlen,
 
 bool mysqld_help(THD *thd, const char *mask)
 {
-  Protocol *protocol= thd->protocol;
-  SQL_SELECT *select;
+  Protocol *protocol= thd->get_protocol();
   st_find_field used_fields[array_elements(init_used_fields)];
-  TABLE_LIST *leaves= 0;
   TABLE_LIST tables[4];
   List<String> topics_list, categories_list, subcategories_list;
   String name, description, example;
-  int count_topics, count_categories, error;
-  uint mlen= strlen(mask);
+  int count_topics, count_categories;
+  size_t mlen= strlen(mask);
   size_t i;
   MEM_ROOT *mem_root= thd->mem_root;
+  SELECT_LEX *const select_lex= thd->lex->select_lex;
   DBUG_ENTER("mysqld_help");
 
   tables[0].init_one_table(C_STRING_WITH_LEN("mysql"),
@@ -692,22 +691,17 @@ bool mysqld_help(THD *thd, const char *mask)
 
   /*
     HELP must be available under LOCK TABLES. 
-    Reset and backup the current open tables state to
-    make it possible.
   */
-  Open_tables_backup open_tables_state_backup;
-  if (open_system_tables_for_read(thd, tables, &open_tables_state_backup))
+  if (open_trans_system_tables_for_read(thd, tables))
     goto error2;
 
   /*
     Init tables and fields to be usable from items
     tables do not contain VIEWs => we can pass 0 as conds
   */
-  thd->lex->select_lex.context.table_list=
-    thd->lex->select_lex.context.first_name_resolution_table= &tables[0];
-  if (setup_tables(thd, &thd->lex->select_lex.context,
-                   &thd->lex->select_lex.top_join_list,
-                   tables, &leaves, FALSE))
+  select_lex->context.table_list=
+    select_lex->context.first_name_resolution_table= &tables[0];
+  if (select_lex->setup_tables(thd, tables, false))
     goto error;
   memcpy((char*) used_fields, (char*) init_used_fields, sizeof(used_fields));
   if (init_fields(thd, tables, used_fields, array_elements(used_fields)))
@@ -715,28 +709,30 @@ bool mysqld_help(THD *thd, const char *mask)
   for (i=0; i<sizeof(tables)/sizeof(TABLE_LIST); i++)
     tables[i].table->file->init_table_handle_for_HANDLER();
 
-  if (!(select=
-	prepare_select_for_name(thd,mask,mlen,tables,tables[0].table,
-				used_fields[help_topic_name].field,&error)))
-    goto error;
+  {
+    QEP_TAB_standalone qep_tab_st;
+    QEP_TAB &tab= qep_tab_st.as_QEP_TAB();
+    if (prepare_select_for_name(thd,mask,mlen,tables,tables[0].table,
+                                used_fields[help_topic_name].field,&tab))
+      goto error;
 
-  count_topics= search_topics(thd,tables[0].table,used_fields,
-			      select,&topics_list,
-			      &name, &description, &example);
-  delete select;
+    count_topics= search_topics(thd, &tab, used_fields,
+                                &topics_list,
+                                &name, &description, &example);
+  }
 
   if (count_topics == 0)
   {
-    int UNINIT_VAR(key_id);
-    if (!(select=
-          prepare_select_for_name(thd,mask,mlen,tables,tables[3].table,
-                                  used_fields[help_keyword_name].field,
-                                  &error)))
+    int key_id= 0;
+    QEP_TAB_standalone qep_tab_st;
+    QEP_TAB &tab= qep_tab_st.as_QEP_TAB();
+
+    if (prepare_select_for_name(thd,mask,mlen,tables,tables[3].table,
+                                used_fields[help_keyword_name].field,
+                                &tab))
       goto error;
 
-    count_topics= search_keyword(thd,tables[3].table, used_fields, select,
-                                 &key_id);
-    delete select;
+    count_topics= search_keyword(thd, &tab, used_fields, &key_id);
     count_topics= (count_topics != 1) ? 0 :
                   get_topics_for_keyword(thd,tables[0].table,tables[2].table,
                                          used_fields,key_id,&topics_list,&name,
@@ -747,24 +743,28 @@ bool mysqld_help(THD *thd, const char *mask)
   {
     int16 category_id;
     Field *cat_cat_id= used_fields[help_category_parent_category_id].field;
-    if (!(select=
-          prepare_select_for_name(thd,mask,mlen,tables,tables[1].table,
-                                  used_fields[help_category_name].field,
-                                  &error)))
-      goto error;
+    {
+      QEP_TAB_standalone qep_tab_st;
+      QEP_TAB &tab= qep_tab_st.as_QEP_TAB();
 
-    count_categories= search_categories(thd, tables[1].table, used_fields,
-					select,
-					&categories_list,&category_id);
-    delete select;
+      if (prepare_select_for_name(thd,mask,mlen,tables,tables[1].table,
+                                  used_fields[help_category_name].field,
+                                  &tab))
+        goto error;
+
+      DEBUG_SYNC(thd, "before_help_record_read");
+
+      count_categories= search_categories(thd, &tab, used_fields,
+                                          &categories_list,&category_id);
+    }
     if (!count_categories)
     {
-      if (send_header_2(protocol,FALSE))
+      if (send_header_2(thd, FALSE))
 	goto error;
     }
     else if (count_categories > 1)
     {
-      if (send_header_2(protocol,FALSE) ||
+      if (send_header_2(thd, FALSE) ||
 	  send_variant_2_list(mem_root,protocol,&categories_list,"Y",0))
 	goto error;
     }
@@ -777,22 +777,31 @@ bool mysqld_help(THD *thd, const char *mask)
       Item *cond_cat_by_cat=
 	new Item_func_equal(new Item_field(cat_cat_id),
 			    new Item_int((int32)category_id));
-      if (!(select= prepare_simple_select(thd, cond_topic_by_cat,
-                                          tables[0].table, &error)))
-        goto error;
-      get_all_items_for_category(thd,tables[0].table,
-				 used_fields[help_topic_name].field,
-				 select,&topics_list);
-      delete select;
-      if (!(select= prepare_simple_select(thd, cond_cat_by_cat,
-                                          tables[1].table, &error)))
-        goto error;
-      get_all_items_for_category(thd,tables[1].table,
-				 used_fields[help_category_name].field,
-				 select,&subcategories_list);
-      delete select;
+
+      {
+        QEP_TAB_standalone qep_tab_st;
+        QEP_TAB &tab= qep_tab_st.as_QEP_TAB();
+
+        if (prepare_simple_select(thd, cond_topic_by_cat,
+                                  tables[0].table, &tab))
+          goto error;
+        get_all_items_for_category(thd, &tab,
+                                   used_fields[help_topic_name].field,
+                                   &topics_list);
+      }
+      {
+        QEP_TAB_standalone qep_tab_st;
+        QEP_TAB &tab= qep_tab_st.as_QEP_TAB();
+
+        if (prepare_simple_select(thd, cond_cat_by_cat,
+                                  tables[1].table, &tab))
+          goto error;
+        get_all_items_for_category(thd, &tab,
+                                   used_fields[help_category_name].field,
+                                   &subcategories_list);
+      }
       String *cat= categories_list.head();
-      if (send_header_2(protocol, TRUE) ||
+      if (send_header_2(thd, TRUE) ||
 	  send_variant_2_list(mem_root,protocol,&topics_list,       "N",cat) ||
 	  send_variant_2_list(mem_root,protocol,&subcategories_list,"Y",cat))
 	goto error;
@@ -800,33 +809,39 @@ bool mysqld_help(THD *thd, const char *mask)
   }
   else if (count_topics == 1)
   {
-    if (send_answer_1(protocol,&name,&description,&example))
+    if (send_answer_1(thd, &name, &description, &example))
       goto error;
   }
   else
   {
     /* First send header and functions */
-    if (send_header_2(protocol, FALSE) ||
+    if (send_header_2(thd, FALSE) ||
 	send_variant_2_list(mem_root,protocol, &topics_list, "N", 0))
       goto error;
-    if (!(select=
-          prepare_select_for_name(thd,mask,mlen,tables,tables[1].table,
-                                  used_fields[help_category_name].field,&error)))
+
+    QEP_TAB_standalone qep_tab_st;
+    QEP_TAB &tab= qep_tab_st.as_QEP_TAB();
+
+    if (prepare_select_for_name(thd,mask,mlen,tables,tables[1].table,
+                                used_fields[help_category_name].field,&tab))
       goto error;
-    search_categories(thd, tables[1].table, used_fields,
-		      select,&categories_list, 0);
-    delete select;
+    search_categories(thd, &tab, used_fields,
+		      &categories_list, 0);
     /* Then send categories */
     if (send_variant_2_list(mem_root,protocol, &categories_list, "Y", 0))
       goto error;
   }
+
+  if (thd->killed)
+    goto error;
+
   my_eof(thd);
 
-  close_system_tables(thd, &open_tables_state_backup);
+  close_trans_system_tables(thd);
   DBUG_RETURN(FALSE);
 
 error:
-  close_system_tables(thd, &open_tables_state_backup);
+  close_trans_system_tables(thd);
 
 error2:
   DBUG_RETURN(TRUE);
