@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1997, 2017, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1997, 2018, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -47,7 +47,6 @@ Created 2/25/1997 Heikki Tuuri
 #include "ibuf0ibuf.h"
 #include "log0log.h"
 #include "fil0fil.h"
-
 /*************************************************************************
 IMPORTANT NOTE: Any operation that generates redo MUST check that there
 is enough space in the redo log before for that operation. This is
@@ -137,6 +136,8 @@ row_undo_ins_remove_clust_rec(
 		ut_a(success);
 	}
 
+	row_convert_impl_to_expl_if_needed(btr_cur, node);
+
 	if (btr_cur_optimistic_delete(btr_cur, 0, &mtr)) {
 		err = DB_SUCCESS;
 		goto func_exit;
@@ -190,7 +191,8 @@ row_undo_ins_remove_sec_low(
 				pessimistic descent down the index tree */
 	dict_index_t*	index,	/*!< in: index */
 	dtuple_t*	entry,	/*!< in: index entry to remove */
-	que_thr_t*	thr)	/*!< in: query thread */
+	que_thr_t*	thr,	/*!< in: query thread */
+	undo_node_t*	node)	/*!< in: undo node */
 {
 	btr_pcur_t		pcur;
 	btr_cur_t*		btr_cur;
@@ -200,7 +202,6 @@ row_undo_ins_remove_sec_low(
 	ibool			modify_leaf = false;
 
 	log_free_check();
-	memset(&pcur, 0, sizeof(pcur));
 
 	mtr_start(&mtr);
 	mtr.set_named_space(index->space);
@@ -255,6 +256,8 @@ row_undo_ins_remove_sec_low(
 
 	btr_cur = btr_pcur_get_btr_cur(&pcur);
 
+	row_convert_impl_to_expl_if_needed(btr_cur, node);
+
 	if (modify_leaf) {
 		err = btr_cur_optimistic_delete(btr_cur, 0, &mtr)
 			? DB_SUCCESS : DB_FAIL;
@@ -285,14 +288,15 @@ row_undo_ins_remove_sec(
 /*====================*/
 	dict_index_t*	index,	/*!< in: index */
 	dtuple_t*	entry,	/*!< in: index entry to insert */
-	que_thr_t*	thr)	/*!< in: query thread */
+	que_thr_t*	thr,	/*!< in: query thread */
+	undo_node_t*	node)
 {
 	dberr_t	err;
 	ulint	n_tries	= 0;
 
 	/* Try first optimistic descent to the B-tree */
 
-	err = row_undo_ins_remove_sec_low(BTR_MODIFY_LEAF, index, entry, thr);
+	err = row_undo_ins_remove_sec_low(BTR_MODIFY_LEAF, index, entry, thr, node);
 
 	if (err == DB_SUCCESS) {
 
@@ -303,7 +307,7 @@ row_undo_ins_remove_sec(
 retry:
 	err = row_undo_ins_remove_sec_low(
 		BTR_MODIFY_TREE | BTR_LATCH_FOR_DELETE,
-		index, entry, thr);
+		index, entry, thr, node);
 
 	/* The delete operation may fail if we have little
 	file space left: TODO: easiest to crash the database
@@ -426,7 +430,7 @@ row_undo_ins_remove_sec_rec(
 			assume that the secondary index record does
 			not exist. */
 		} else {
-			err = row_undo_ins_remove_sec(index, entry, thr);
+			err = row_undo_ins_remove_sec(index, entry, thr, node);
 
 			if (UNIV_UNLIKELY(err != DB_SUCCESS)) {
 				goto func_exit;
